#!/bin/bash
# lliurex dhcp & dns funcs
# creado por Luis García para LliureX en Abril 2006

# variables
MAC_DATABASE="/etc/lliurex-mac.db"
MAC_CFGDIR="dhcp/mac"
ALIAS_CFGDIR="dns/alias"
ALIAS_FILE="llxcfg"

NET_VAR_LIST="HOSTNAME_PREFIX DHCP_HOST_MAX DHCP_UNREG_HOST_MAX SRV_DOMAIN"
[ -x "/usr/sbin/llxcfg-showvars" ] && eval `/usr/sbin/llxcfg-showvars ${NET_VAR_LIST}`

# comprobaciones
LIB_FILE="/usr/share/lliurex/lliurex-netfuncs/lliurex-netfuncs.sh"
[ -e "$LIB_FILE" ] && . $LIB_FILE

# BEGIN LLIUREX SPECIFIC NETWORK FUNCTIONS

get_dhcp_range(){
   # Devuelve el rango de direcciones "libres" para ofrecer por dhcp
   # Usa la función split_network a partir de la IP del servidor,
   # pero aumentando en 1 bit la máscara, con lo que subdivide
   # la semired del servidor, es decir, se queda con 1/4 de la red original que
   # ni incluye al servidor ni se solapa con el rango de reservas del DHCP.
   #
   SRV_IP="`get_srv_ip`"
   SRV_BITS="`get_srv_bitmask`"
   NEW_NET="`split_network ${SRV_IP}/$((${SRV_BITS}+1))`"
   SRV_FIRST="`ipcalc_hostmin ${NEW_NET}`"
   SRV_LAST="`ipcalc_hostmax ${NEW_NET}`"
   if [ "$DHCP_UNREG_HOST_MAX" ] ; then
      SRV_BITS="`get_srv_bitmask`"
      UNREG_HOST_MAX=$((2**(30-$SRV_BITS)-2))
      if [ $UNREG_HOST_MAX -gt $DHCP_UNREG_HOST_MAX ] ; then
         NUM_IP=$((`ip2num ${SRV_FIRST}/8` + $DHCP_UNREG_HOST_MAX -1))
         SRV_LAST="`num2ip ${NUM_IP}`"
      fi
   fi
   echo "${SRV_FIRST} ${SRV_LAST}"
}

get_dhcp_host_address(){
   # argumentos: HOST_NUM1 [HOST_NUM2]
   # Devuelve la lista de direcciones de red correspondiente a los host (con IP fija) que
   # ocupan el rango entre la posición HOST_NUM1 y HOST_NUM2 en las reservas de DHCP
   # La numeración de hosts comienza en el 1
   # Para establecer el rango de direcciones se utiliza split_network a partir de
   # la IP del servidor para dividir la red "por la mitad" en dos subredes y
   # encontrar la "mitad" en la que no se encuentra la dirección del servidor
   # Después se utiliza ipcalc_addnum2ip, lo que garantiza (y limita) que las
   # direcciones no excedan el intervalo hostmin,hostmax de la semired correspondiente.
   #
   SRV_IP="`get_srv_ip`"
   SRV_BITS="`get_srv_bitmask`"
   FIRST_HOST="`split_network ${SRV_IP}/${SRV_BITS}`"
   HOST_NUM="`dhcp_host_num $1`"
   IP_NUM=`ip2num ${FIRST_HOST}`
   IP_NUM=$((${IP_NUM} + ${HOST_NUM} -1))
   if [ "$2" ]; then
      HOST_NUM2=`dhcp_host_num $2`
   else
      HOST_NUM2=${HOST_NUM}
   fi
   IP_NUM2=$(($IP_NUM + $HOST_NUM2 - $HOST_NUM))
   
   while [ ${IP_NUM} -le ${IP_NUM2} ]; do
      echo "`num2ip ${IP_NUM}`"
      IP_NUM=$((${IP_NUM}+1))
   done
}

get_dhcp_host_max(){
   SRV_BITS="`get_srv_bitmask`"
   MAX_HOSTS=$((2**(31-$SRV_BITS)-2))
   if [ $MAX_HOSTS -lt $DHCP_HOST_MAX ] ; then
      echo $MAX_HOSTS
   else
      echo $DHCP_HOST_MAX
   fi
}

dhcp_host_num(){
   local HOST_NUM HOST_ARG HMAX
   HMAX=$(get_dhcp_host_max)
   HOST_ARG="$1"
   HOST_NUM=$((${HOST_ARG#0}))
   [ ${HOST_NUM} -lt 0 ] && HOST_NUM=0
#   [ ${HOST_NUM} -gt ${DHCP_HOST_MAX} ] && HOST_NUM=${DHCP_HOST_MAX}
   [ ${HOST_NUM} -gt ${HMAX} ] && HOST_NUM=${HMAX}
   echo ${HOST_NUM}
}
dhcp_host_string(){
   local HOST_NUM HOST_ARG
   HOST_ARG="$1"
   HOST_NUM=`dhcp_host_num ${HOST_ARG#0}`
   [ ${HOST_NUM} -lt 10 ] && HOST_NUM="0${HOST_NUM}"
   echo ${HOST_NUM}
}

get_dhcp_host_name(){
   # argumentos: HOST_NUM
   # Devuelve el nombre de host correspondiente a la reserva que ocupa la posición
   # HOST_NUM (comenzando por el 1) en las reservas de DHCP
   # Los nombres se construyen concatenando el prefijo "HOSTNAME_PREFIX" y el número
   # HOST_NUM expresado con 2 cifras (lo que fija el límite en 99 hosts)
   #
   HOST_NUM="`dhcp_host_string $1`"
   if [ "${HOST_NUM}" = "00" ]; then
      echo "`get_srv_name`"
   else
      echo "${HOSTNAME_PREFIX}${HOST_NUM}"
   fi
}

get_dhcp_host_num(){
   # argumentos: HOST_NAME
   # Devuelve el número de host a partir de su nombre
   # si el nombre no se corresponde con un host registrado (o el server) devuelve
   # una cadena vacía
   #
   HOST_NAME="$1"
   if [ "${HOST_NAME}" = "`get_srv_name`" ]; then
      echo 0
      return 0
   fi
   HOST_STR="${HOST_NAME#${HOSTNAME_PREFIX}}"
   HOST_NUM=$((${HOST_STR}))
   if [ ${HOST_NUM} -gt 0 ]; then
      echo ${HOST_NUM}
   else
      echo ""
   fi
}

test_dhcp_db(){
   return 0
}

set_dhcp_host_mac(){
   HOST_NUM="`dhcp_host_string $1`"
   HOST_MAC="$(get_normalized_mac_address "$2")" || return 0
   shift 2
   HOST_ALIAS="$@"
   echo "$HOST_MAC" | llxcfg-config write "${MAC_CFGDIR}/${HOST_NUM}"
   (llxcfg-config dump "${ALIAS_CFGDIR}/${HOST_NUM}" ; echo "$HOST_ALIAS" |tr " " "\n") |sort -u |llxcfg-config write "${ALIAS_CFGDIR}/${HOST_NUM}/${ALIAS_FILE}"
}

del_dhcp_host_mac(){
   HOST_NUM="`dhcp_host_string $1`"
   llxcfg-config delete "${MAC_CFGDIR}/${HOST_NUM}"
}

get_dhcp_host_mac(){
   HOST_NUM="`dhcp_host_string $1`"
   llxcfg-config read "${MAC_CFGDIR}/${HOST_NUM}" || true
}

get_dhcp_all_alias(){
   echo $(llxcfg-config -n dump "${ALIAS_CFGDIR}")
}

get_dhcp_host_alias(){
   HOST_NUM="`dhcp_host_string $1`"
   echo $(llxcfg-config -u -n dump "${ALIAS_CFGDIR}/${HOST_NUM}" )
}

add_dhcp_host_alias(){
   HOST_NUM="`dhcp_host_string $1`"
   shift
   HOST_ALIAS="$@"
   NEW_ALIAS="`get_dhcp_host_alias ${HOST_NUM}`"
   ALL_ALIAS="`get_dhcp_all_alias`"
   for a in ${HOST_ALIAS}; do
      if ! echo " ${NEW_ALIAS} ${ALL_ALIAS} " |grep -q " ${a} "; then
         NEW_ALIAS="${NEW_ALIAS} ${a}"
      fi
   done
   replace_dhcp_host_alias ${HOST_NUM} ${NEW_ALIAS}  
}

replace_dhcp_host_alias(){
   local HOST_NUM NEW_ALIAS TMP_FILE
   HOST_NUM="`dhcp_host_string $1`"
   shift
   NEW_ALIAS="$@"
   echo "$NEW_ALIAS" |tr " " "\n" |llxcfg-config write "${ALIAS_CFGDIR}/${HOST_NUM}/${ALIAS_FILE}"
}

del_dhcp_host_alias(){
   HOST_NUM="`dhcp_host_string $1`"
   shift
   HOST_ALIAS="$@"
   NEW_ALIAS=""
   CURRENT_ALIAS="`get_dhcp_host_alias ${HOST_NUM}`"
   for a in ${CURRENT_ALIAS}; do
      if ! echo " ${HOST_ALIAS} " |grep -q " ${a} "; then
         NEW_ALIAS="${NEW_ALIAS} ${a}"
      fi
   done
   replace_dhcp_host_alias ${HOST_NUM} ${NEW_ALIAS}
}

get_dhcp_host_list(){
#   IP_LIST="`get_dhcp_host_address 1 ${DHCP_HOST_MAX}`"
   IP_LIST="`get_dhcp_host_address 1 $(get_dhcp_host_max)`"
   HOST_NUM=1
   for HOST_IP in ${IP_LIST}; do
#      HOST_ID="`dhcp_host_string ${HOST_NUM}`"
      HOST_NAME="`get_dhcp_host_name ${HOST_NUM}`"
      HOST_MAC="`get_dhcp_host_mac ${HOST_NUM}`"
      HOST_ALIAS="`get_dhcp_host_alias ${HOST_NUM}`"
      echo "${HOST_NAME},${HOST_IP},${HOST_MAC},${HOST_ALIAS},;"
      HOST_NUM=$((${HOST_NUM}+1))
   done
}

get_dhcp_dnsmasq_mac_list(){
   for f in $(llxcfg-config list "${MAC_CFGDIR}") ; do
      F="${f#/}"
      echo "dhcp-host=$(llxcfg-config read "${MAC_CFGDIR}/$f"),$(get_dhcp_host_address "$F"),${HOSTNAME_PREFIX}$F"
   done
}

get_dhcp_dnsmasq_list(){
   IP_LIST="`get_dhcp_host_address 1 $(get_dhcp_host_max)`"
   HOST_NUM=1
   for HOST_IP in ${IP_LIST}; do
      HOST_STR="$HOST_NUM"
      [ $HOST_NUM -gt 9 ] || HOST_STR="0$HOST_STR"
      echo "$HOST_IP ${HOSTNAME_PREFIX}${HOST_STR}.$SRV_DOMAIN"
      HOST_NUM=$((${HOST_NUM}+1))
   done
}

get_dhcp_dnsmasq_alias_list(){
   IP_LIST="`get_dhcp_host_address 1 $(get_dhcp_host_max)`"
   HOST_NUM=1
   ALL_ALIAS="$(get_dhcp_host_alias 0)"
   for HOST_IP in ${IP_LIST}; do
      HOST_STR="$HOST_NUM"
      [ $HOST_NUM -gt 9 ] || HOST_STR="0$HOST_STR"
      HOST_ALIAS="$(llxcfg-config -n -u dump "${ALIAS_CFGDIR}/${HOST_STR}")"
      if [ "$HOST_ALIAS" ] ; then
         HOST_NAME="${HOSTNAME_PREFIX}${HOST_STR}"
         for alias in ${HOST_ALIAS}; do
            if ! echo " $ALL_ALIAS " |grep -q " $alias " ; then
               echo "cname=${alias}.$SRV_DOMAIN,$HOST_NAME.$SRV_DOMAIN"
               ALL_ALIAS="$ALL_ALIAS $alias"
            fi
         done
      fi
      HOST_NUM=$((${HOST_NUM}+1))
   done
}

get_noreg_dnsmasq_list(){
   # devuelve la lista completa de IPs dinámicas
   HOST_RANGE="`get_dhcp_range`"
   NUM_IP="`ip2num ${HOST_RANGE%%\ *}/8`"
   NUM_IP2="`ip2num ${HOST_RANGE##*\ }/8`"
   while [ ${NUM_IP} -le ${NUM_IP2} ]; do
      HOST_IP="`num2ip ${NUM_IP}`"
      HOST_NAME="`get_noreg_host_name ${HOST_IP}`"
      echo "${HOST_IP} ${HOST_NAME}.$SRV_DOMAIN"
      NUM_IP=$((${NUM_IP}+1))
   done

}

get_noreg_host_list(){
   # devuelve la lista completa de IPs dinámicas
   HOST_RANGE="`get_dhcp_range`"
   NUM_IP="`ip2num ${HOST_RANGE%%\ *}/8`"
   NUM_IP2="`ip2num ${HOST_RANGE##*\ }/8`"
   while [ ${NUM_IP} -le ${NUM_IP2} ]; do
      HOST_IP="`num2ip ${NUM_IP}`"
      HOST_NAME="`get_noreg_host_name ${HOST_IP}`"
      echo "${HOST_NAME},${HOST_IP},;"
      NUM_IP=$((${NUM_IP}+1))
   done
}


is_valid_mac_address(){
   echo $1 | egrep -q  "([0-9a-fA-F]+[:-]){5}([0-9a-fA-F])"  && return 0 || return 1
}

get_normalized_mac_address(){
   ret=0
   is_valid_mac_address $1 || ret=1
   if [ ${ret} -eq 1 ] ; then
      echo ""
      return 1
   else
      echo -n $1 | tr "\-:[a-f]" "\n\n[A-F]" | sed -e "s%^\([0-9A-F]\)$%0\1%g" | tr  "\n" ":" 
   fi
   return 0 
}


# END LLIUREX SPECIFIC NETWORK FUNCTIONS


